@@ -140,7 +140,11 @@ class SnowflakeIdentifierPreparer(compiler.IdentifierPreparer):
                 pre_idx += 1
         if pre_idx < idx:
             ret.append(schema[pre_idx:idx])
-        return ret
+
+        # convert the returning strings back to quoted_name types, and assign the original 'quote' attribute on it
+        quoted_ret = [quoted_name(value, quote=getattr(schema, 'quote', None)) for value in ret]
+
+        return quoted_ret
 
 
 class SnowflakeCompiler(compiler.SQLCompiler):
@@ -424,6 +428,24 @@ class SnowflakeDDLCompiler(compiler.DDLCompiler):
                  in file_format.formatter.options.items()])
         )
 
+    def visit_drop_table_comment(self, drop, **kw):
+        """Snowflake does not support setting table comments as NULL.
+
+        Reflection has to account for this and convert any empty comments to NULL.
+        """
+        table_name = self.preparer.format_table(drop.element)
+        return "COMMENT ON TABLE {} IS ''".format(table_name)
+
+    def visit_drop_column_comment(self, drop, **kw):
+        """Snowflake does not support directly setting column comments as NULL.
+
+        Instead we are forced to use the ALTER COLUMN ... UNSET COMMENT instead.
+        """
+        return "ALTER TABLE {} ALTER COLUMN {} UNSET COMMENT".format(
+            self.preparer.format_table(drop.element.table),
+            self.preparer.format_column(drop.element)
+        )
+
 
 class SnowflakeTypeCompiler(compiler.GenericTypeCompiler):
     def visit_BYTEINT(self, type_, **kw):
@@ -483,6 +505,8 @@ class SnowflakeTypeCompiler(compiler.GenericTypeCompiler):
     def visit_TIMESTAMP(self, type_, **kw):
         return "TIMESTAMP"
 
+    def visit_GEOGRAPHY(self, type_, **kw):
+        return "GEOGRAPHY"
 
 construct_arguments = [
     (Table, {
