@@ -4,6 +4,7 @@
 # Copyright (c) 2012-2019 Snowflake Computing Inc. All right reserved.
 #
 
+from collections import defaultdict
 import operator
 from functools import reduce
 
@@ -44,7 +45,7 @@ from .base import (
     SnowflakeIdentifierPreparer,
     SnowflakeTypeCompiler,
 )
-from .custom_types import ARRAY, OBJECT, TIMESTAMP_LTZ, TIMESTAMP_NTZ, TIMESTAMP_TZ, VARIANT
+from .custom_types import ARRAY, OBJECT, TIMESTAMP_LTZ, TIMESTAMP_NTZ, TIMESTAMP_TZ, VARIANT, GEOGRAPHY
 
 colspecs = {}
 
@@ -82,15 +83,19 @@ ischema_names = {
     'VARIANT': VARIANT,
     'OBJECT': OBJECT,
     'ARRAY': ARRAY,
+    'GEOGRAPHY': GEOGRAPHY,
 }
 
 
 class SnowflakeDialect(default.DefaultDialect):
     name = 'snowflake'
     driver = 'snowflake'
-    max_identifier_length = 65535
+    max_identifier_length = 255
     cte_follows_insert = True
 
+    # TODO: support SQL caching, for more info see: https://docs.sqlalchemy.org/en/14/core/connections.html#caching-for-third-party-dialects
+    supports_statement_cache = False
+
     encoding = UTF8
     default_paramstyle = 'pyformat'
     colspecs = colspecs
@@ -271,6 +276,11 @@ class SnowflakeDialect(default.DefaultDialect):
         # no index is supported by Snowflake
         return []
 
+    @reflection.cache
+    def get_check_constraints(self, connection, table_name, schema, **kw):
+        # check constraints are not supported by Snowflake
+        return []
+
     @reflection.cache
     def _get_schema_primary_keys(self, connection, schema, **kw):
         result = connection.execute(text(
@@ -280,7 +290,7 @@ class SnowflakeDialect(default.DefaultDialect):
         for row in result:
             table_name = self.normalize_name(row['table_name'])
             if table_name not in ans:
-                ans[table_name] = {'constrained_columns': [], 'name': None}
+                ans[table_name] = {'constrained_columns': [], 'name': self.normalize_name(row['constraint_name'])}
             ans[table_name]['constrained_columns'].append(self.normalize_name(row['column_name']))
         return ans
 
@@ -296,8 +306,43 @@ class SnowflakeDialect(default.DefaultDialect):
         ).get(table_name, {'constrained_columns': [],
                            'name': None})
 
+    @reflection.cache
+    def _get_schema_unique_constraints(self, connection, schema, **kw):
+        result = connection.execute(text(
+            "SHOW /* sqlalchemy:_get_schema_unique_constraints */ UNIQUE KEYS IN SCHEMA {0}".format(schema)
+        ))
+        unique_constraints = {}
+        for row in result:
+            name = self.normalize_name(row['constraint_name'])
+            if name not in unique_constraints:
+                unique_constraints[name] = {
+                    'column_names': [self.normalize_name(row['column_name'])],
+                    'name': name,
+                    'table_name': self.normalize_name(row['table_name'])
+                }
+            else:
+                unique_constraints[name]['column_names'].append(self.normalize_name(row['column_name']))
+
+        ans = defaultdict(list)
+        for constraint in unique_constraints.values():
+            table_name = constraint.pop('table_name')
+            ans[table_name].append(constraint)
+        return ans
+
+    def get_unique_constraints(self, connection, table_name, schema, **kw):
+        schema = schema or self.default_schema_name
+        current_database, current_schema = self._current_database_schema(connection, **kw)
+        full_schema_name = self._denormalize_quote_join(
+            current_database, schema if schema else current_schema)
+        return self._get_schema_unique_constraints(
+            connection,
+            self.denormalize_name(full_schema_name),
+            **kw
+        ).get(table_name, [])
+
     @reflection.cache
     def _get_schema_foreign_keys(self, connection, schema, **kw):
+        _, current_schema = self._current_database_schema(connection, **kw)
         result = connection.execute(text(
             "SHOW /* sqlalchemy:_get_schema_foreign_keys */ IMPORTED KEYS IN SCHEMA {0}".format(schema)
         ))
@@ -305,9 +350,14 @@ class SnowflakeDialect(default.DefaultDialect):
         for row in result:
             name = self.normalize_name(row['fk_name'])
             if name not in foreign_key_map:
+                referred_schema = self.normalize_name(row['pk_schema_name'])
                 foreign_key_map[name] = {
                     'constrained_columns': [self.normalize_name(row['fk_column_name'])],
-                    'referred_schema': self.normalize_name(row['pk_schema_name']),
+                    # referred schema should be None in context where it doesn't need to be specified
+                    # https://docs.sqlalchemy.org/en/14/core/reflection.html#reflection-schema-qualified-interaction
+                    'referred_schema': (referred_schema
+                                        if referred_schema not in (self.default_schema_name, current_schema)
+                                        else None),
                     'referred_table': self.normalize_name(row['pk_table_name']),
                     'referred_columns': [self.normalize_name(row['pk_column_name'])],
                     'name': name,
